extends KinematicBody2D

const SPEED : int = 200
const GRAVITY : int = 1200
const UP : Vector2 = Vector2.UP
const JUMP_HEIGHT : int = -330

onready var raycasts = get_node("Raycasts")

var is_lower : bool
var motion : Vector2 = Vector2()
var is_holding : bool = false
var is_grounded : bool = true

func _ready() -> void:
	set_physics_process(true)

func _physics_process(delta : float) -> void:
	motion.y += GRAVITY * delta
	if Global.progress["game_progress"] == 1:
		_get_input(SPEED / 4, false)
		is_lower = true
	elif Global.progress["game_progress"] == 2:
		_get_input(SPEED, false)
		is_lower = false
	else:
		_get_input(SPEED, true)
		is_lower = false

	motion = move_and_slide(motion, UP)
	is_grounded = _check_is_grounded()
	_assign_animation()
	_change_collision()

func _change_collision() -> void:
	if is_lower:
		$CollisionShape2D.disabled = true
		$CollisionLower.disabled = false
	else:
		$CollisionShape2D.disabled = false
		$CollisionLower.disabled = true

func _get_input(velocity : int, jump : bool) -> void:
	var move_direction
	move_direction = -int(Input.is_action_pressed("ui_left")) + int(Input.is_action_pressed("ui_right"))
	
	_change_direction(move_direction)

	motion.x = lerp(motion.x, velocity * move_direction, 1)

	if jump and Input.is_action_just_pressed("ui_up") and is_grounded:
		motion.y = JUMP_HEIGHT

func _change_direction(direction : int) -> void:
	if direction == -1:
		$Sprite.flip_h = true
	elif direction == 1:
		$Sprite.flip_h = false

func _check_is_grounded() -> bool:
	for raycast in raycasts.get_children():
		if raycast.is_colliding():
			return true

	return false

func _assign_animation() -> void:
	var animation 
	
	if is_lower:
		animation = "Idle_lower"
	else:
		animation = "Idle"
	
	if !is_grounded:
		animation = "Jump"
	else:
		if is_holding:
				animation = "Holding"
		elif motion.x != 0:
			if is_lower:
				animation = "Walk_lower"
			else:
				animation = "Walk"
			
	$Sprite.play(animation)